package com.xy.service;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.yitter.idgen.YitIdHelper;
import com.xy.utils.AliOssUtil;
import com.xy.utils.R;
import com.xy.vo.PolicyVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.*;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/***
 * 对象存储实现类
 * @author 谭斌
 * @date 2023/1/9 11:22
 */
@Api(tags = "对象存储")
@Slf4j
@Service
@RequiredArgsConstructor
public class OssServiceImpl implements OssService {

    private final AliOssService aliOssService;

    @ApiOperation(value = "policy获取", notes = "阿里云OSS服务端签名后前端直传方式获取policy")
    @GetMapping("aliOSS/policy")
    public R<PolicyVo> getPolicy(@ApiParam(name = "dir", value = "上传目录", required = false) @RequestParam(value = "dir", required = false) String dir) {
        return aliOssService.getPolicy(dir);
    }

    @ApiOperation(value = "获取指定目录下的文件列表", notes = "获取指定目录下的文件列表")
    @GetMapping("aliOSS/listFilesInDir")
    public R<List<AliOssUtil.FileInfo>> listFilesInDir(@ApiParam(name = "dir", value = "目录", required = false) @RequestParam(value = "dir", required = false) String dir) {
         List<AliOssUtil.FileInfo> files = AliOssUtil.listFilesInDir(dir, null);
        files.forEach(file -> {
            System.out.println(String.format("%s | 大小: %dB | 修改时间: %tF",
                    file.getFileName(),
                    file.getSize(),
                    file.getUpdateTime()));
        });

        return R.ok(files);
    }

    /**
     *指定目录上传文件
     * @param fileUrl   /存放目录/文件名.后缀
     * @param file
     * @return
     */
    @ApiOperation("指定目录上传文件")
    @PostMapping("aliOSS/uploadFile")
    public R<String> uploadFile( @RequestParam("fileUrl") String fileUrl, @RequestParam("file") MultipartFile file) {
        try (InputStream inputStream = file.getInputStream()) {

            String url = AliOssUtil.upload(inputStream, fileUrl);
            return R.ok(url);
        } catch (IOException e) {
            log.error("文件处理时发生错误: {}", e.getMessage());
            return R.fail("文件处理时发生错误");
        }
    }


}
